import businessLogic.ppg.XpShopCode
import tangram.ConfigFile
import tangram.DataBuffer
import tangram.Log
import tangram.SQLconn
import tangram.SQLtable
import tangram.http.TangramHttpRequest

// CUSTOMERCREATE vlozenie noveho zakaznika
urlCustomerCreate(sql, request, cfg)

// Zoznam skladov (predajni) - exportuju sa vsetky okrem velkoskladov
private static void urlCustomerCreate (SQLconn sql, TangramHttpRequest request, ConfigFile cfg)
{
    // vstupne data su v request.bodyMap (rozparsovany JSON)
    def body = request.bodyMap
    def firma
    def customerType = body.getInt('customer_type', 0)

    // Kontrola a generovanie kodu FIRMA (vetvenie 1=B2B, inak B2C)
    if (customerType == 1)
    {
        // B2B zakaznik (Firma), ICO sa pouzije ako kod firmy
        firma = body.getString('siret')
        // id = firma
        if (firma.isEmpty())
        {
            request.sendError(firma, 'SIRETNULL', "Nezadané IČO pre firmu ${body.getString('company_first_name')}")
            return
        }
        def ico = body.getBigDecimalOrNull('siret')
        // Log.debug("SIRET char=$firma number=$ico")
        if (ico == null)
        {
            request.sendError(firma, 'SIRETBAD', "IČO musí byť číslo $firma")
            return
        }
        // test, ci firma uz existuje
        def nazov = sql.queryValueStr("SELECT Nazov1 FROM FIRMY WHERE Xx='$firma'")
        if (!nazov.isEmpty())
        {
            request.sendError(firma, 'EXIST', "IČO $firma už existuje v databáze (${nazov})")
            return
        }
    }
    else
    {
        // B2C zakaznik, firma="Ennnn", poradove cislo pridelena systemom MAX+1, zarovnane na 6 cislic
        def mxx = sql.queryValueStr("SELECT max(Firma) FROM FIRMY WHERE translate(Firma, '0123456789', '0000000000') = 'ESK-000000'")
        def mnn = 0
        if (mxx)
        {
            mnn = mxx.substring(4).toInteger() + 1
            if (mnn > 999999)
            {
                request.sendError('', 'B2CMAX', "Kód zákazníka ESK-nnnnnn presiahol 6 číslic")
                return
            }
        }
        firma = 'ESK-' + mnn.toString().padLeft(6,'0')
    }

    // Kontrola e-mailu
    if (!checkEmail(body.getString('email'), sql, request)) return

    // Nastavenie poli
    Log.debug ("CUSTOMER_CREATE: $body")
    def f = new SQLtable ("FIRMY", sql)
    f.xx        = firma
    f.firma     = firma
    parseBody(sql, f, body, cfg)
    f.insert()
    f.close()

    // koniec OK
    Log.info("NEW CUSTOMER, id=$firma, name=${f.nazov1}, XP=${f.xp}")
    request.sendOk(firma, 'Zákazník pridaný do databázy')
}

// Kontrola EMAILu
private static boolean checkEmail (String email, SQLconn sql, TangramHttpRequest request)
{
    // EMAIL: musi byt zadany, musi obsahovat @, nesmie obsahovat medzery alebo ciarky, musi byt unikatny (FIRMY.FLG_ESHOP)
    if (email.isEmpty())
    {
        request.sendError('', 'EMAILNULL', "Nezadaný E-mail (povinná položka)")
        return false
    }
    if ((!email.contains('@')) || email.contains(' ') || email.contains(','))
    {
        request.sendError('', 'EMAILBAD', "E-mail chybný formát ($email)")
        return false
    }
    def xx = sql.queryValueStr("SELECT max(Xx) as Xx FROM FIRMY WHERE Flg_eshop='$email'")
    if (!xx.isEmpty())
    {
        request.sendError(xx, 'EMAILEXIST', "E-mail $email už existuje (zákazník ${xx})")
        return false
    }
    return true
}

// Naplnenie stlpcov tabulky FIRMY podla requestu
private static void parseBody(SQLconn sql, SQLtable f, DataBuffer body, ConfigFile cfg)
{
    // body.customer_type: 1 = B2B, else=B2C
    def customerType = body.getInt('customer_type', 0)
    def priceGroupDefault = cfg.str('EshopPriceGroup', '?')
    def shopDefault = cfg.str('EshopShop', '?')
    def eshopCountry = cfg.str('EshopCountry')
    def isCompany = (customerType == 1)

    f.f         = isCompany ? 'P' : 'O'    // Pravnicka osoba / Obcan
    f.do        = 'O'                      // odberatel
    f.neaktivna = isCompany ? 'B' : null   // B=nova firma na schvalenie / null=aktivna firma
    //f.neaktivna = null                  // TEST: kazda nova firma je hned schvalena
    f.ico       = isCompany ? body.siret : null
    f.x_crmexport = 'A'                 // Zapnuty export do ClientLink

    // NAZOV pre firmu je rozdeleny do dvoch poli, meno obcana sa zlozi Firstname+Lastname (30.08.2022)
    if (isCompany)
    {
        f.nazov1     = body.company_first_name
        f.nazov2     = body.company_last_name
    }
    else
    {
        f.nazov1 = body.delivery_firstName + ' ' + body.delivery_lastName
        f.nazov2 = null
    }

    // DICDU, DIC, DANURAD
	f.icdph		 = body.vat ? body.vat : '-'
	f.dicdu      = body.vat ? body.vat : '-'

    f.ulica      = body.invoice_addr1 + (body.invoice_addr2 ? ' ' + body.invoice_addr2 : '')
    f.mesto      = body.invoice_city
    f.psc        = body.invoice_post
    f.stat       = body.invoice_country
    f.druh_statu = (eshopCountry != body.invoice_country) ? 'B' : 'A'
    //OKRES, KRAJ, KLUC1, KLUC2, KLUC3, FAX
    f.telefon    = body.phone
    f.email      = body.email
    f.kraj = 'BA'            // dočasně doplněná výchozí hodnota 10, aby se v testu dalo hned posílat dál do CL
    //OBCHODNÍ PODMÍNKY
    f.zlavatyp   = 'B'       // Firemna zlava (%)
    f.zlava      = 3         // Predefined discount
    // f.zakaz_f    = 'A'    // Zakaz fakturacie
    f.spuhrady   = 'H,D,V,X'    // Povolene platby			upraveno 16.9. na výchozí hodnotu očekávanou webem
    f.censkup    = body.getString('price_group', priceGroupDefault)     // Predefined price group
    f.xp         = XpShopCode.removePrefix(body.getString("default_shop_id"), shopDefault)
    f.kluc1      = f.xp     // doplněno
    f.kluc3      = 'A99'     // doplněno 6.12.2022
    f.flg_eshop  = body.email
    f.x_crmid    = body.ClientLinkProspectID                            // doplněno znovu 31.8.2022
	
    // Obchodny zastupca: prednostne sa pouzije z requestu, inak sa prenasa z tabulky ADRESA, stlpec "VEDUCI" (aj ked to nie je urcene na obch., zastupcu)
    f.obchzast = body.sales_rep_id
    if (! body.sales_rep_id)
        f.obchzast = sql.queryValueStr("SELECT Veduci FROM ADRESA WHERE Xx='${f.xp}'")

    // pre B2B sa pouzije "Invoice company name" ak nie je zadane "Delivery company name"
    if (isCompany)
	{
	    f.odbskup = body.customer_group
        f.delivery_company = body.delivery_company ?: body.company_first_name + ' ' + body.company_last_name
	}
    else
	{
	    f.odbskup = "N"
        f.delivery_company = body.delivery_company
	}

    f.delivery_firstname = body.delivery_firstname
    f.delivery_lastname = body.delivery_lastname
    f.delivery_addr1    = body.delivery_addr1
    f.delivery_addr2    = body.delivery_addr2
    f.delivery_post     = body.delivery_post
    f.delivery_city     = body.delivery_city
    f.delivery_country  = body.delivery_country
}