//import businessLogic.PpgOrder
//import businessLogic.PpgOrderHead
//import businessLogic.PpgOrderRow


import businessLogic.SkPrice
import businessLogic.ppg.PpgOrderRowP
import businessLogic.ppg.PpgOrderRowT
import tangram.ConfigFile
import tangram.JsonWriter
import tangram.Log
import tangram.SQLconn
import tangram.http.TangramHttpRequest

import java.time.LocalDate

// PRICE - vyhladanie ceny pre jeden produkt (ak je zadane pole COLORANTS, je to miesana farba
// parametre su v tele requestu (POST); CUSTOMER_ID, ITEM_ID a pole COLORANTS
urlPrice (sql, request, cfg)

// Pomocna trieda pre naformatovanie vysledku (Jackson z toho generuje JSON)
class PriceResult
{
    public String result = 'OK'
    public String customer_name
    public String price_group
    public BigDecimal discount
    public String item_id
    public BigDecimal priceb_bv
    public BigDecimal priceb_iv
    public BigDecimal priced_bv
    public BigDecimal priced_iv
    public BigDecimal vat
    public String promotion
    public BigDecimal price_discount
    public String price_type
    public LocalDate valid_from
    public LocalDate valid_to
    public String comment

    public void loadPrice (SkPrice price)
    {
        priceb_bv  = price.cena_bdph
        priceb_iv  = price.cena_sdph
        priced_bv  = price.zlav_bdph
        priced_iv  = price.zlav_sdph
        vat        = price.sadzba_dph
        promotion  = price.akcia
        price_discount = price.zlava
        price_type = price.typceny
        valid_from = price.datum_od
        valid_to   = price.datum_do
        comment    = price.popis
    }
}

private static void urlPrice (SQLconn sql, TangramHttpRequest request, ConfigFile cfg)
{
    // Header: podla kodu zakaznika vyhladame Cenovu skupinu a Dokladovu zlavu
    def body = request.bodyMap
    def customerId = body.getString('customer_id')
    def itemId = body.getString('item_id')
    if (!itemId) { request.sendError('', 'IDNULL', "ITEM ID not specified"); return }
    def colorants = body['colorants']
    def priceGroupDefault = cfg.str('EshopPriceGroup', '001')
    def priceGroupOverride = cfg.bool("EshopPriceOverride")

    // Vyhladanie zakaznika, ak sa nenajde pouziju sa default hodnoty
    // simulujeme OrderHead, lebo cez hlavicku sa odovzdaju parametre na hladanie ceny
    def firma = sql.querySingle("SELECT * FROM FIRMY WHERE Xx='$customerId'")
    // parametre pre vypocet ceny
    def prm = new SkPrice.SkPricePrm();
    prm.pKod = itemId
    prm.pDatum = LocalDate.now()
    prm.pFirma = firma.getString('firma')
    prm.pPobocka = firma.getString('pobocka')
    prm.pCenskup = priceGroupOverride ? priceGroupDefault : (firma.getString('censkup') ?: priceGroupDefault)
    prm.pZlDoklad = firma.getBigDecimal('zlava')
    prm.pPredajna = firma.getString('xp')

    // Miesto pre vysledok operacie, predplnia sa niektore polia
    def result = new PriceResult()
    result.customer_name = firma.getString('nazov1')
    result.price_group = prm.pCenskup
    result.discount = prm.pZlDoklad
    result.item_id = itemId

    // simulujeme OrderRow, bud produkt alebo miesana farba
    Log.debug("PRICE: item=$itemId customer=$customerId pricegroup=${prm.pCenskup} cust.discount=${prm.pZlDoklad} xp=${prm.pPredajna}")
    if (!colorants)
    {
        def row = new PpgOrderRowP()
        row.item_id = itemId
        row.loadPrice(sql, prm)
        // row.recalcPrice(zl_f)
        // Kontrola chyb pre PRODUCT, nenajdena cena
        if (!row.price.found)
        {
            request.sendError(itemId, 'PRICENULL', "ITEM ${itemId}, group ${prm.pCenskup}, price not found")
            return
        }
        Log.debug(row.price.toString())
        result.loadPrice(row.price)
    }
    else
    {
        def row = new PpgOrderRowT()
        row.item_id = itemId
        row.parseColorants(sql, colorants)
        row.loadPrice(sql, prm)
        // row.recalcPrice(zl_f)
        // Kontrola cien
        if (!row.basePrice.found) { request.sendError(itemId, 'PRICENULL', "COLOR BASE ${itemId}, price not found"); return }
        for (c in row.colorants)
        {
            if (c.sku.isEmpty()) { request.sendError(itemId,'COLIDNULL',"COLORANT ${c.colorId} not found in table (X_COLORCODE)"); return }
            if (!c.price.found) { request.sendError(itemId,'COLPRICENULL',"COLORANT ${c.colorId} (${c.sku}), price not found"); return }
        }
        Log.debug(row.price.toString())
        result.loadPrice(row.price)
    }

    // Response header + body
    request.sendHeaderJSON()
    JsonWriter.write(request.writer, result)
}


