import tangram.ConfigFile
import tangram.SQLconn
import tangram.TangramJsonWriter
import tangram.http.TangramHttpRequest

// QUOTE DETAIL: cenova ponuka (OBP) hlavicka + pole riadkov, JSON, UTF-8
// args: id = OBCPH.XX
urlQuoteDetail(sql, request, json, cfg)

private static void urlQuoteDetail(SQLconn sql, TangramHttpRequest request, TangramJsonWriter json, ConfigFile cfg)
{
    def id = request.queryMap.getString('id')

    // Response header
    request.sendHeaderJSON()

    // Reponse body
    json.begin()
    exportHeader (sql, json, id, cfg)
    exportRows (sql, json, id)
    json.end()
}

// OBCPH -> JSON
private static void exportHeader (SQLconn sql, TangramJsonWriter json, String id, ConfigFile cfg)
{
    // select je zhodny s QuoteList
    def country = cfg.str("EshopCountry")
    def select = """
        SELECT H.Xx "quote_id", 
            H.Druhdok || '/' || to_char(H.Cdok) "quote_visible_id",
            H.Druhdok "quote_group",
            H.Cdok "quote_number",
            H.Firmaxx "customer_id",
            H.Censkup "pricegroup",
            H.Zl_f "discount",
            H.Datum "date",
            H.Popis "comment",
            '$country' || H.Xp "delivery_shop",
            H.Delivery_company,
            H.Delivery_firstname,
            H.Delivery_lastname,
            H.Delivery_addr1,
            H.Delivery_addr2,
            H.Delivery_post,
            H.Delivery_city,
            H.Delivery_country,
            H.Delivery_email,
            H.Delivery_phone,
            R.Totalb_bv,
            R.Totalb_iv,
            R.Totald_bv,
            R.Totald_iv
        FROM OBCPH H LEFT JOIN 
        ( SELECT Xh, sum(round(Cm_jedn * Mn, 2)) as Totalb_bv, sum(round(Cd_jedn * Mn, 2)) as Totalb_iv, sum(Cm_suma) as Totald_bv, sum(Cd_suma) as Totald_iv
          FROM OBCPR WHERE Xh='$id' GROUP BY Xh
        ) R ON (R.Xh=H.Xx) 
        WHERE H.Xx='$id'
    """.stripMargin()
    def data = sql.querySingle (select)
    json.map(data, "header")
}

// OBPH -> JSON
private static void exportRows (SQLconn sql, TangramJsonWriter json, String id)
{
    // select je zhodny s QuoteList
    def select = """
        SELECT R.Xn "rownum",
            R.Kod as item_id,
            R.Popis "comment",
            nvl(R.Znazov, T.Nazov) as name,
            T.Jkoef as package,
            T.Jmj as package_unit,
            R.Mn as quantity,
            R.Cm_jedn as Priceb_bv,
            R.Cd_jedn as Priceb_iv,
            R.Cm_bdph as Priced_bv,
            R.Cd_cena as Priced_iv,
            round(R.Cm_jedn * R.Mn, 2) as Totalb_bv,
            round(R.Cd_jedn * R.Mn, 2) as Totalb_iv,
            R.Cm_suma as Totald_bv,
            R.Cd_suma as Totald_iv,
            R.Zlava as Discount,
            R.Sadzba_dph as Vat
        FROM OBCPR R LEFT JOIN TOVARY T ON (T.Xx=R.Kod)
        WHERE R.Xh='$id' and (r.blok='H' or r.blok is null) ORDER BY R.Xn
    """.stripMargin()
    def rs = sql.queryRows (select)
    json.resultset(rs, "rows")
    rs.close()
}
