import tangram.BigNumber
import tangram.DataBuffer
import tangram.Log

// SURFPOINT PAYMENT
new Payment().urlPayment(sql, request, json, cfg)

class Payment
{
    private BigDecimal sadzba_dph = 20      // pouzije sa pre priamu platbu
    private BigDecimal platba_cash = 0
    private BigDecimal platba_card = 0
    private String firma = ''

    // Vstupne parametre: request (request+response), sql (tangram.SQLConn), query (parametre requestu), body (parsovane telo requestu), logger
    public void urlPayment (tangram.SQLconn sql, tangram.http.TangramHttpRequest request, tangram.TangramJsonWriter json, tangram.ConfigFile cfg)
    {
        // Priprava dat pre hlavicku, zapis bude az na konci (potrebujeme najprv spocitat sumy)
        def body    = request.bodyMap
        def header  = body.getObject('header')
        def vats    = body.getArray('vats')
        def pays    = body.getArray('payments')
        Log.debug ("PAYMENT: $header")

        // ID dokladu sa pouzije ako PCKASADOKH.XX (ak sa nezada, vygenerujeme nase XX)
        // XX sa overi v databaze, aby zapis nepadol do Unique violation (DPH a platby by uz boli zapisane, hlavicka nie)
        // Ak uz doklad existuje, nic sa nezapisuje, ale vratime odpoved OK ako keby sme ho uspesne vlozili
        def pckasadokhxx = header.getString('id', sql.createXX())
        def exist = sql.queryValueInt("SELECT count(*) FROM PCKASADOKH WHERE Xx='$pckasadokhxx'")
        if (!exist)
        {
            // objednavka: kod firmy sa preberie z objednavky aby sa urobil samostatny skl. doklad a vyparovala sa objednavka
            // priama platba: kod firmy zostane prazdny, vytvori sa kumulovany doklad
            def obphxx = header.getString('orderid')
            def sumVat = loadVAT(sql, pckasadokhxx, vats)
            def sumPay = loadPayments(sql, pckasadokhxx, pays)
            if (obphxx) loadRowOrder(sql, pckasadokhxx, obphxx)
            else loadRowSimple(sql, pckasadokhxx, header.getString('itemid'), sumVat)

            saveHeader(sql, pckasadokhxx, header, sumVat)

            if (obphxx) markOrder(sql, obphxx)
        }

        // Response header nezapisujeme, urobi to za nas handler po navrate
        // Reponse body: zapisujeme len do bufferu, nesmieme urobit flush, to za nas urobi handler
        json.begin()
        json.pair("result", "OK")
        json.pair("id", pckasadokhxx)
        json.end()
    }

    // ROZPIS DPH, nacita a zapise do databazy, vrati sucet s DPH
    // Naplni premennu [sadzba_dph]
    private BigDecimal loadVAT (tangram.SQLconn sql, String xx, List<DataBuffer> vats)
    {
        // VATS = rozpis DPH (array of maps), vypocita sumu za doklad
        def table = sql.prepareTable ('PCKASADOKD', false)
        BigDecimal suma = 0
        vats.eachWithIndex { vat, index ->
            Log.debug("VAT: $vat")
            table.clear()
            table.xx = sql.createXX()
            table.xh = xx
            table.xn = index+1
            table.xc = 1
            table.flg_dph = 'A'
            table.sadzba_dph = vat.vat
            sadzba_dph = vat.vat
            table.sdph = vat.amountinc
            table.dph = vat.amountvat
            table.bdph = (vat.amountinc - vat.amountvat)
            suma += vat.amountinc
            table.insert()
        }
        table.close()
        return suma
    }

    // PAYMENTS = rozpis platidiel (array), nacita a zapise do databazy, vrati sucet platieb
    // Naplni premenne [platba_cash] a [platba_card]
    private BigDecimal loadPayments (tangram.SQLconn sql, String xx, List<DataBuffer> payments)
    {
        def table = sql.prepareTable ("PCKASADOKP", false)
        BigDecimal platby = 0
        payments.eachWithIndex { pay, index ->
            Log.debug("PAY: $pay")
            table.clear()
            table.xx = sql.createXX()
            table.xh = xx
            table.xn = index+1
            table.xc = 1
            table.platbaxx = pay.type
            if (pay.type == 'CARD')
            {
                table.platbadruh='B'
                platba_card += pay.amount
            }
            else
            {
                table.platbadruh='$'
                platba_cash += pay.amount
            }
            table.suma = pay.amount
            table.popis = pay.text
            platby += pay.amount
            table.insert()
        }
        table.close()
        return platby
    }

    // OBJEDNAVKA: vytvorit riadky podla objednavky, oznacit objednavku za vybavenu (?)
    // Naplni premennu [firma]
    private void loadRowOrder (tangram.SQLconn sql, String xx, String obphxx)
    {
        def pcr = sql.prepareTable ('PCKASADOKR')
        def obph = sql.querySingle ("SELECT * FROM OBPH WHERE Xx='$obphxx'")
        def obpr = sql.queryRows ("SELECT * FROM OBPR WHERE Xh='$obphxx' ORDER BY Xn")
        firma = obph.getString('firma')
        while (obpr.next())
        {
            println("OBPR next: $obpr.xx $obpr.xn")
            pcr.clear()
            pcr.xh = xx
            pcr.xn = obpr.xn
            pcr.obpxx = obpr.xx
            pcr.kod = obpr.kod
            pcr.mn  = obpr.mn
            pcr.cennik = obpr.cm_sdph
            pcr.cena_t = obpr.cm_sdph
            pcr.cena = obpr.cm_sdph
            pcr.suma = (obpr.cm_sdph * obpr.mn).round(2)
            pcr.sadzba_dph = obpr.sadzba_dph
            pcr.flg_dph = 'A'       // rezim DPH "standar"
            pcr.censkup = obph.censkup
            pcr.znazov = obpr.znazov
            pcr.poznamka = pcr.popis
            pcr.str = obpr.str
            pcr.zak = obpr.zak
            pcr.kp  = obpr.kp
            pcr.insert()
        }
        pcr.close()
        obpr.close()
        // obph.close() sa neuzatvara, je to DataBuffer
    }

    // ROW = riadok dokladu, len ak je to priama platba (kod sluzby)
    private void loadRowSimple (tangram.SQLconn sql, String xx, String itemid, BigDecimal suma)
    {
        def pcr = sql.prepareTable ('PCKASADOKR')
        pcr.clear()
        pcr.xh = xx
        pcr.xn = 1
        pcr.kod = itemid
        pcr.mn  = 1
        pcr.cennik = suma
        pcr.cena_t = suma
        pcr.cena = suma
        pcr.suma = suma
        pcr.sadzba_dph = sadzba_dph  // z rozpisu DPH
        pcr.censkup = 'S001'         // zatial natvrdo
        pcr.flg_dph = 'A'            // rezim DPH "standard"
        pcr.insert()
        pcr.close()
    }

    // PCKASADOKH: vlozenie hlavicky PCkasa doklad
    private void saveHeader (tangram.SQLconn sql, String xx, DataBuffer header, BigDecimal suma)
    {
        // HEADER = hlavicka PCKASA dokladu
        def pch = sql.prepareTable ('PCKASADOKH')
        pch.clear()
        pch.xx = xx
        pch.sklad = header.device
        pch.kasa = header.device
        pch.typdokladu = 'A'    // A (standardny doklad)
        pch.datum = header.getDate('date', java.time.LocalDate.now())
        pch.cas = header.getTimeMinutes('time')
        pch.cdok = header.receipt
        pch.suma = suma
        pch.zaokr = BigDecimal.ZERO
        pch.nsuma = suma
        pch.dsuma = BigDecimal.ZERO
        pch.p_hotovost = (platba_cash != 0) ? platba_cash : null
        pch.p_banka = (platba_card != 0) ? platba_card : null
        pch.fik = header.uid
        pch.bkp = header.okp
        pch.firma = firma
        pch.poznamka = "${header.text} Order=${header.orderid} Item=${header.itemid}"     // sem dat xx+cislo objednavky
        pch.flg_odpis = null
        pch.insert()
        pch.close()
    }

    // OBPH: ak bola uhrada objednavky, oznacime objednavku za "Vybavenu"
    private void markOrder (tangram.SQLconn sql, String obphxx)
    {
        Log.debug("ORDER $obphxx: označuje sa ako VYBAVENÁ")
    }
}

// --- SCRIPT END ---
