import com.sun.deploy.security.SelectableSecurityManager
import tangram.DataBuffer
import tangram.DateFunctions

// SURFPOINT PAYMENT
// Vstupne parametre: request (request+response), sql (tangram.SQLConn), query (parametre requestu), body (parsovane telo requestu), logger
// def data = new DataBuffer()

// Priprava dat pre hlavicku, zapis bude az na konci (potrebujeme najprv spocitat sumy)
def header = body.header
logger.debug ("PAYMENT: $header")
def pckasadokhxx = sql.createXX()
def objednavka = (header.orderid != null)

// VATS = rozpis DPH (array of maps), vypocita sumu za doklad
def table = sql.prepareTable ("PCKASADOKD")
table.xMode = false
def suma = 0
body.vats.eachWithIndex { vat, index ->
    logger.debug("VAT: $vat")
    table.clear()
    table.xx = sql.createXX()
    table.xh = pckasadokhxx
    table.xn = index+1
    table.xc = 1
    table.flg_dph = 'A'
    table.sadzba_dph = vat.vat
    table.sdph = vat.amountinc
    table.dph = vat.amountvat
    table.bdph = (vat.amountinc - vat.amountvat)
    suma += vat.amountinc
    table.insert()
}
table.close()

// PAYMENTS = rozpis platidiel (array)
table = sql.prepareTable ("PCKASADOKP")
table.xMode = false
def platby = 0
def platba_cash = 0
def platba_card = 0
body.payments.eachWithIndex { pay, index ->
    logger.debug("PAY: $pay")
    table.clear()
    table.xx = sql.createXX()
    table.xh = pckasadokhxx
    table.xn = index+1
    table.xc = 1
    table.platbaxx = pay.type
    if (pay.type == 'CARD')
        {
            table.platbadruh='B'
            platba_card += pay.amount
        }
    else
        {
            table.platbadruh='$'
            platba_cash += pay.amount
        }
    table.suma = pay.amount
    table.popis = pay.text
    platby += pay.amount
    table.insert()
}
table.close()

// HEADER = hlavicka PCKASA dokladu
table = sql.prepareTable ("PCKASADOKH")
table.clear()
table.xx = pckasadokhxx
table.sklad = header.device
table.kasa = header.device
table.typdokladu = objednavka ? 'H' : 'A'    // A (priama platba) alebo H (objednavka)
table.datum = header.getDate("date", java.time.LocalDate.now())
table.cas = header.getTimeMinutes("time")
table.cdok = header.receipt  //sql.queryMaxPlus('PCKASADOKH', 'cdok', 'filter na kasu a dnesny den')
table.suma = suma
table.zaokr = 0
table.p_hotovost = (platba_cash != 0) ? platba_cash : null
table.p_banka = (platba_card != 0) ? platba_card : null
table.fik = header.uid
table.bkp = header.okp
// table.hotovost = (suma - platby)
// table.firma = 'KIOSK'        // OBPH.Firma ak je objednavka
table.poznamka = "${header.text} Order=${header.orderid} Item=${header.itemid}"     // sem dat xx+cislo objednavky
table.flg_odpis = objednavka ? "OBJEDNAVKA" : null      // priznak, aby sa uz nerobil dodaci list
table.insert()
table.close()

// ROW = riadok dokladu, len ak je to priama platba (kod sluzby)
if (!objednavka)
{
    table = sql.prepareTable ("PCKASADOKR")
    table.clear()
    table.xh = pckasadokhxx
    table.xn = 1
    table.kod = header.itemid
    table.mn  = 1
    table.cennik = suma
    table.cena_t = suma
    table.cena = suma
    table.suma = suma
    table.sadzba_dph = 20       // zatial natvrdo
    table.censkup = "S01"       // zatial natvrdo
    table.insert()
    table.close()
}

// OBJEDNAVKA: vytvorit DL podla objednavky (OBPH->SKLADH, OBPR->SKLADR), vlozit PCKASA.XX -> SKLADH.DDXX
if (objednavka)
{
    createDlFromObp (header, pckasadokhxx)
}

// Response header nezapisujeme, urobi to za nas handler po navrate
// Reponse body: zapisujeme len do bufferu, nesmieme urobit flush, to za nas urobi handler
json.begin()
json.pair("result", "OK")
json.pair("id", pckasadokhxx)
json.end()

// --- SCRIPT END ---

// Funkcia: vytvorenie DL z prijatej objednavky (vsetky riadky, cele objednane mnozstvo) a oznacenie objednavky "vybavena"
def createDlFromObp (reqHeader, ddxx)
{
    logger.debug("CREATE DL, OBPXX=$reqHeader.orderid")

    // Nacitat hlavicku (buffer) a riadky objednavky (resultset)
    def obphPrm = new tangram.DataBuffer()
    obphPrm.xx = reqHeader.orderid
    def obph = sql.querySingle ("SELECT * FROM OBPH WHERE Xx=:xx", obphPrm)
    def obpr = sql.queryRows ("SELECT * FROM OBPR WHERE Xh=:xx ORDER BY Xn", obphPrm)

    // Hlavicka skladoveho dokladu
    def skladh = sql.prepareTable ("SKLADH")
    def skladhxx = sql.createXX()
    def today = java.time.LocalDate.now()
    def yearBegin = today.withDayOfYear(1)
    def dateFunc = new DateFunctions()
    def sqlYearBegin = dateFunc.toSQL(yearBegin)
    skladh.xx = skladhxx
    skladh.xp = obph.xp
    skladh.kategoria = 'D'      // Predaj
    skladh.druhdok = reqHeader.device
    skladh.cdok = sql.queryMaxPlus ("SKLADH", "CDOK", "Datum >= $sqlYearBegin")
    skladh.datum = today
    skladh.ddtyp = 'B'          // SKL_DDTYP_PCKASA	= "B"
    skladh.ddxx = ddxx
    skladh.firma = obph.firma
    skladh.rsstav = '1'         // Doklad ukonceny
    skladh.popis = "Kiosk: úhrada objednávky $obph.druhdok $obph.cdok"
    skladh.insert()
    skladh.close()

    // Riadky skladoveho dokladu
    def skladr = sql.prepareTable ("SKLADR")
    while (obpr.next())
    {
        println("OBPR next: $obpr.xx $obpr.xn")
        skladr.clear()
        skladr.xh = skladhxx
        skladr.xn = obpr.xn
        skladr.druhdok = skladh.druhdok // finta: vytiahnutie parametrov z tabulky SKLADH (funguje aj po zatvoreni)
        skladr.cdok = skladh.cdok
        skladr.datum = skladh.datum
        skladr.kod = obpr.kod
        skladr.kategoria = 'S'      // Vzdy SLUZBA
        skladr.mena = obpr.mena
        skladr.cm_cenn = obpr.cm_cenn
        skladr.cm_jedn = obpr.cm_jedn
        skladr.cm_bdph = obpr.cm_bdph
        skladr.cm_sdph = obpr.cm_sdph
        skladr.uc_cenn = obpr.cm_cenn
        skladr.uc_jedn = obpr.cm_jedn
        skladr.uc_bdph = obpr.cm_bdph
        skladr.uc_sdph = obpr.cm_sdph
        skladr.cm_cenn = obpr.cm_cenn
        skladr.cm_jedn = obpr.cm_jedn
        skladr.cm_bdph = obpr.cm_bdph
        skladr.sadzba_dph = obpr.sadzba_dph
        skladr.censkup = obph.censkup
        skladr.mnp = 0
        skladr.mnv = obpr.mn
        skladr.obpxx = obpr.xx
        skladr.popis = obpr.popis
        skladr.znazov = obpr.znazov
        skladr.str = obpr.str
        skladr.zak = obpr.zak
        skladr.kp  = obpr.kp
        skladr.flg_dph = 'A'        // DPH standard tuzemsko
        skladr.insert()
    }
    skladr.close()
    obpr.close()

    // Oznacit objednavku "VYBAVENA" ihned. DL obsahuje odkaz na OBP, takze zostane vybavena aj po preparovani.
    // sql.execute ("UPDATE OBPH SET Vyba='A' WHERE Xx=:xx", obphPrm)
    sql.execute ("UPDATE OBPR SET Mndod=Mn WHERE Xh=:xx", obphPrm)
}
