import businessLogic.SkUtils
import tangram.SQLconn
import tangram.SQLtable
import tangram.DataBuffer
import tangram.TangramJsonWriter
import tangram.http.TangramHttpRequest

// vlozenie e-shop zakaznika B2C
// Vstupne parametre: request (request+response), query (parametre requestu), body (parsovane telo requestu)
// sql (tangram.SQLConn), json (JSON writer), log (Logger)
urlCustomerCreate (sql, request, json, cfg)

// Zoznam skladov (predajni) - exportuju sa vsetky okrem velkoskladov
private static void urlCustomerCreate (SQLconn sql, TangramHttpRequest request, TangramJsonWriter json, tangram.ConfigFile cfg)
{
    def body = request.bodyMap
    def f = sql.prepareTable("FIRMY")

    formatFirma(f, body, cfg)
    def firmaXx = SkUtils.firmaJoin(f.firma, f.pobocka)
    f.xx = firmaXx

    // kontrola
    if (!firmaXx) { request.sendError('', 'EMPTY', 'Nezadaný kód zákazníka (id)'); return }
    def exist = sql.queryValueInt("SELECT count(*) FROM FIRMY WHERE Xx='$firmaXx'")
    if (exist)  { request.sendError(firmaXx, 'EXIST', "Zákazník s kódom $firmaXx už existuje v databáze"); return }

    // zapis do databazy
    f.insert()
    f.close()

    // Response header + body
    request.sendOk(firmaXx)
}

// Vyplnenie stlpcov tabulky FIRMY
private static Boolean formatFirma (SQLtable f, DataBuffer body, tangram.ConfigFile cfg)
{
    // kod zakaznika a dalsie
    def firmaXx = body.getString('id')
    f.xx        = firmaXx
    f.firma     = SkUtils.firmaFirma(firmaXx)
    f.pobocka   = SkUtils.firmaPobocka(firmaXx)
    // Forma spolocnosti: mozno pouzit B2B=P/B2C=O, alebo priamo hodnoty P/F/I/O
    switch(body.typ)
    {
        case 'B2B': f.f = 'P'; break;
        case 'P':   f.f = 'P'; break;
        case 'F':   f.f = 'F'; break;
        case 'I':   f.f = 'I'; break;
        default:    f.f = 'O'; break;
    }
    f.do        = 'O' // Odberatel
    f.ico       = body.ico
    f.dicdu     = body.dicdu
    f.icdph     = body.getString('icdph', '-')
    f.druh_statu = body.getString('druh_statu', 'A')   // Tuzemsko
    // hlavna adresa
    f.nazov1    = body.nazov1
    f.nazov2    = body.nazov2
    f.ulica     = body.ulica
    f.psc       = body.psc
    f.mesto     = body.mesto
    f.stat      = body.getString('stat', cfg.str('EshopCountry'))
    f.okres     = body.okres
    f.kraj      = body.kraj
    f.adresa    = body.adresa
    f.poznamka  = body.poznamka     // max 250
    // dodacia adresa
    f.flg_eshop         = body.getString('login', 'A')
    f.xp                = body.getString('delivery_shop', cfg.str('EshopShop'))
    f.delivery_company  = body.delivery_company
    f.delivery_firstname = body.delivery_firstname
    f.delivery_lastname = body.delivery_lastname
    f.delivery_addr1    = body.delivery_addr1
    f.delivery_addr2    = body.delivery_addr2
    f.delivery_post     = body.delivery_post
    f.delivery_city     = body.delivery_city
    f.delivery_country  = body.delivery_country
    // telefon, email a dalsie udaje
    f.telefon   = body.telefon
    f.email     = body.email
    f.zlavatyp  = 'B'
    f.zlava     = body.zlava
    f.censkup   = body.getString('censkup', cfg.str('EshopPriceGroup', '001'))
    f.odbskup   = body.odbskup
    f.trasa     = body.trasa
    f.obchzast  = body.obchzast
    f.kluc1     = body.kluc1
    f.kluc2     = body.kluc2
    f.kluc3     = body.kluc3
    f.dnis      = body.dnis
    f.ss        = body.ss
    f.default_dl = body.default_dl
    f.default_df = body.default_df
    f.skalias    = body.skalias
    f.c_kodbuctu = body.c_kodbuctu

    // Sposob uhrady: ak sa vobec nezada, pouzije 'P,H,D'. Moze sa data aj prazdna hodnota = zakaz fakturacie.
    f.spuhrady = (body.containsKey('spuhrady')) ? body.spuhrady : 'P,H,D'
}
