import tangram.Log
import tangram.SQLconn
import tangram.TangramJsonWriter
import tangram.http.TangramHttpRequest

// zoznam zakaznikov (GET), query-parametre:
// [id]=kod zakaznika (firmy.xx), vrati jeden zaznam (ID ma prednost pred ostatnymi parametrami)
// [days]=pocet dni od poslednej zmeny, vrati pole vsetkych zmenenych zakaznikov (0=zmenene dnes, 1=zmenene od vcera rana, ...)
// [filter] = SQL filter na tabulku FIRMY bez aliasu
// specialne pripady: ziaden parameter exportuje vsetkych zakaznikov zmenenych za poslednych 999 dni
urlCustomerList (sql, request, json, cfg)

// Zoznam zakaznikov
private static void urlCustomerList (SQLconn sql, TangramHttpRequest request, TangramJsonWriter json, tangram.ConfigFile cfg)
{
    // parametre requestu
    def params = request.queryMap
    def id = params.getString('id')
    def days = params.getInt('days', -1)
    def filter = params.getString('filter')

    // SELECT
    def sqlFilter = createFilter (id, days, filter)
    Log.debug("filter: $sqlFilter")
    def rs = sql.queryRows("SELECT * FROM FIRMY WHERE $sqlFilter ORDER BY Xx")

    // Response header
    request.sendHeaderJSON()

    // formatovanie dat: ak je ID tak zakaznik bude v jednom bloku, inak bude obsah zabaleny do pola "customers"
    if (id)
    {
        rs.next()
        exportJSON(rs.data, json)
    }
    else
    {
        json.begin()
        json.arrayBegin('customers')
        while (rs.next())
            exportJSON(rs.data, json)
        json.arrayEnd()
        json.end()
    }

    Log.debug("rows: ${rs.getRow()}")
    rs.close()
}

// Vytvorenie SQL filtra pre select z tabulky FIRMYU
private static String createFilter (String id, int days, String filter)
{
    // ID ma prednost - exportuje sa jediny zakaznik, ziadne dalsie filtre sa neaplikuju
    if (id) return "Xx='$id'"

    def sqlFilter = new tangram.SQLfilter()
    sqlFilter.add("Flg_eshop is not null")
    if (days >= 0) sqlFilter.add("nvl(Xud,Xid) >= (trunc(sysdate)-$days)")
    if (filter) sqlFilter.add(filter)
    return sqlFilter.toString()
}

// Formatovanie jedneho zakaznika do JSON (ak sa nenajde, budu vsetky polia "null")
private static void exportJSON(tangram.DataBuffer data, TangramJsonWriter json)
{
    json.blockBegin()

    json.pair('id',         data.xx)
    json.pair('firma',      data.firma)
    json.pair('pobocka',    data.pobocka)
    json.pair('typ',        (data.f=='O') ? 'B2C' : 'B2B')    // B2B, B2C
    json.pair('ico',        data.ico)
    json.pair('dicdu',      data.dicdu)
    json.pair('icdph',      data.icdph)
    json.pair('druh_statu', data.druh_statu)
    // sidlo firmy - hlavna adresa
    json.pair('nazov1',     data.nazov1)
    json.pair('nazov2',     data.nazov2)
    json.pair('ulica',      data.ulica)
    json.pair('mesto',      data.mesto)
    json.pair('psc',        data.psc)
    json.pair('stat',       data.stat)
    json.pair('okres',      data.okres)
    json.pair('kraj',       data.kraj)
    json.pair('adresa',     data.adresa)
    json.pair('poznamka',   data.poznamka)
    // adresa pobocky - dodacia adresa
    json.pair('login',              data.flg_eshop)
    json.pair('delivery_shop',      data.xp)
    json.pair('delivery_company',   data.delivery_company)
    json.pair('delivery_firstname', data.delivery_firstname)
    json.pair('delivery_lastname',  data.delivery_lastname)
    json.pair('delivery_addr1',     data.delivery_addr1)
    json.pair('delivery_addr2',     data.delivery_addr2)
    json.pair('delivery_post',      data.delivery_post)
    json.pair('delivery_city',      data.delivery_city)
    json.pair('delivery_country',   data.delivery_country)
    // ostatne
    json.pair('telefon',    data.telefon)
    json.pair('email',      data.email)
    json.pair('zlava',      data.zlava)
    json.pair('censkup',    data.censkup)
    json.pair('odbskup',    data.odbskup)
    json.pair('trasa',      data.trasa)
    json.pair('obchzast',   data.obchzast)
    json.pair('kluc1',      data.kluc1)
    json.pair('kluc2',      data.kluc2)
    json.pair('kluc3',      data.kluc3)
    // fakturacia
    json.pair('spuhrady',   data.spuhrady)
    json.pair('dnis',       data.dnis)
    json.pair('ss',         data.ss)
    json.pair('default_dl', data.default_dl)
    json.pair('default_df', data.default_df)

    json.blockEnd()
}
