import tangram.Log
import tangram.DataBuffer
import businessLogic.SkUtils

// vlozenie objednavky, obsah je v mape body
// Vstupne parametre: request (request+response), sql (tangram.SQLConn), query (parametre requestu)
new Order().urlOrderCreate(sql, request, json, cfg)

class Order
{
    // Spolocne premenne medzi OBPH a OBPR
    private String sklad

    // Vstupny bod
    public void urlOrderCreate (tangram.SQLconn sql, tangram.http.TangramHttpRequest request, tangram.TangramJsonWriter json, tangram.ConfigFile cfg)
    {
        // analyza requestu
        def header  = request.bodyMap.getObject('header')
        def rows    = request.bodyMap.getArray('rows')
        Log.debug("OBPH: $header")
        Log.debug("ROWS: $rows")
        // >> TODO: skontrolovat, ci existuje header aj rows, inak by sme mali vratit chybu volajucemu (cez Exception ?)

        // vlozenie do databazy
        def obphxx = sql.createXX()
        def info = createHeader(sql, header, obphxx, cfg)
        createRows(sql, rows, obphxx)

        // rezervacia mnozstva
        reservation(sql, obphxx)

        // Notifikacia o novej objednavke do TN_MSG
        businessLogic.TN_MSG.write(sql, "OrderNotification", obphxx)
        // Notifikacia o platbe vopred (mala by sa vystavit "zalohova" faktura)
        if (header.sp_uhrady == 'V')
            businessLogic.TN_MSG.write(sql, "OrderPayment", obphxx)

        // HTTP response
        request.sendOk(obphxx, info)
    }

    // Zapis hlavicky do databazy, vracia OBPH.XX
    // Ak sa uvedie CDOK, pouzije sa ako interne cislo (inak sa pocita MAX+1)
    // Ak sa uvedie EXTCIS, pouzije sa externe cislo (inak sa dosadi EXTCIS=CDOK)
    private String createHeader (tangram.SQLconn sql, DataBuffer header, String obphxx, tangram.ConfigFile cfg)
    {
        def customerid  = header.getString('customer', '0').take(17)
        def druhdok     = header.getString('druhdok','ESHOP')
        def cdok        = header.getBigNumber('cdok')
        if (cdok.isNull()) cdok = sql.queryMaxPlus('OBPH', 'cdok', "Druhdok='$druhdok'")
        def firma = sql.querySingle("SELECT * FROM FIRMY WHERE Xx='$customerid'")

        def obph = sql.prepareTable ("OBPH")

        obph.xx         = obphxx
        obph.druhdok    = druhdok
        obph.cdok       = cdok
        obph.extcis     = header.getString('extcis', cdok.toString())
        obph.datum      = header.getDate('datum', java.time.LocalDate.now())
        // Stav objednavky podla konfiguracie (EshopOrderClose=false necha objednavku otvorenu)
        obph.rsstav     = header.stav ?: cfg.bool('EshopOrderClose',true) ? '1' : '0'

        // obph.delivery_shop = header.delivery_shop
        obph.firma      = firma.firma
        obph.pobocka    = firma.pobocka
        obph.trasa      = header.trasa ?: firma.trasa
        obph.d_termin   = header.getDate('d_termin') ?: obph.datum
        obph.t_termin   = header.getTimeMinutes('t_termin')
        obph.popis      = header.popis
        obph.text       = header.text
        obph.firmaref   = header.delivery_ref   // delivery reference
        obph.referent   = header.referent       // nas referent, ktory vybavuje objednavku

        // predvolena predajna, z nej nacita sklad, sklad sa prenesie cez class-field do createRows
        def xp          = header.delivery_shop ?: firma.xp ?: cfg.str('EshopShop')
        sklad           = sql.queryValueStr("SELECT Sklad FROM ADRESA WHERE Xx='$xp'")

        obph.xp         = xp
        obph.sklad      = sklad
        obph.censkup    = header.censkup ?: firma.censkup ?: cfg.str('EshopPriceGroup')
        obph.zl_f       = header.containsKey('zlava') ? header.getBigNumber('zlava') : firma.zlava
        obph.obchzast   = header.obchzast ?: firma.obchzast
        obph.spedicia   = header.spedicia
        obph.sp_uhrady  = header.sp_uhrady ?: 'D'   // default DOBIERKA
        obph.vs_uhrady  = header.vs_uhrady          // VS k uhrade vopred cez platobnu branu

        // Delivery adresa prednostne z webu, ak nie je pouzije sa z adresara firiem
        def delivery = header.delivery_company || header.delivery_firstname || header.delivery_lastname || header.delivery_addr1 || header.delivery_addr2
        obph.delivery_company   = delivery ? header.delivery_company : firma.delivery_company
        obph.delivery_firstname = delivery ? header.delivery_firstname : firma.delivery_firstname
        obph.delivery_lastname  = delivery ? header.delivery_lastname : firma.delivery_lastname
        obph.delivery_addr1     = delivery ? header.delivery_addr1 : firma.delivery_addr1
        obph.delivery_addr2     = delivery ? header.delivery_addr2 : firma.delivery_addr2
        obph.delivery_city      = delivery ? header.delivery_city : firma.delivery_city
        obph.delivery_post      = delivery ? header.delivery_post : firma.delivery_post
        obph.delivery_country   = delivery ? header.delivery_country : firma.delivery_country
        obph.delivery_phone     = delivery ? header.delivery_phone : firma.telefon
        obph.delivery_email     = delivery ? header.delivery_email : firma.email

        def obphInserted = obph.insert()
        def info = "Nová objednávka ${obph.druhdok}/${obph.cdok}, zákazník=${customerid}"
        Log.info(info)
        obph.close()

        return info
    }

    // Zapis riadkov do databazy
    private void createRows (tangram.SQLconn sql, ArrayList<DataBuffer> rows, String obphxx)
    {
        // ROWS (array of maps)
        def obpr = sql.prepareTable ("OBPR")
        rows.eachWithIndex { row, index ->
            Log.debug("ROW: $row")
            def mn      = row.getBigNumber('mn')
            obpr.clear()
            obpr.xh     = obphxx
            obpr.xn     = index+1
            obpr.kod    = row.kod
            obpr.popis  = row.popis
            obpr.znazov = row.znazov
            obpr.mn     = mn
            obpr.mnzost = mn
            obpr.sklad  = row.sklad ?: sklad
            obpr.str    = row.str
            obpr.zak    = row.zak
            obpr.kp     = row.kp
            //
            def cm_cenn = row.getBigNumber('cm_cenn')
            def cd_cenn = row.getBigNumber('cd_cenn')
            def cm_cena = row.getBigNumber('cm_cena')
            def cd_cena = row.getBigNumber('cd_cena')
            // cena - ak je zadana aspon jedna cena (bez/s DPH), bude zavazna
            if (cm_cena.nonZero || cd_cena.nonZero)
            {
                def sadzba_dph  = row.getBigNumber('sadzba_dph')
                obpr.zavazna    = 'A'
                obpr.mena       = row.getString('mena', 'EUR')
                obpr.sadzba_dph = sadzba_dph
                obpr.zlava      = row.getBigNumber('zlava')
                obpr.zl_t       = row.getBigNumber('zlava')
                obpr.zl_typ     = 'B'       // len riadkova zlava
                // ak je zadana len jedna zlavnena cena (s/bez DPH), dopocitame opacnu
                if (cm_cena.nullOrZero) cm_cena = SkUtils.cenaMinusDph(cd_cena, sadzba_dph)
                if (cd_cena.nullOrZero) cd_cena = SkUtils.cenaPlusDph(cm_cena, sadzba_dph)
                // ak mame zlavnene ceny a nemame cennikove ceny, tak ich doplnime (v tejto chvili uz musia byt zlavnene ceny zadane)
                if (cm_cenn.nullOrZero) cm_cenn = cm_cena
                if (cd_cenn.nullOrZero) cd_cenn = cd_cena
                // ceny stare aj nove (od 20.03) sa nastavia sucasne, zapise sa ta ktora existuje v tabulke
                obpr.cm_cenn    = cm_cenn
                obpr.cm_jedn    = cm_cenn
                obpr.cm_bdph    = cm_cena
                obpr.cd_cenn    = cd_cenn
                obpr.cd_jedn    = cd_cenn
                obpr.cd_cena    = cd_cena
                obpr.cm_sdph    = cd_cena
                obpr.cm_suma    = (cm_cena * mn).round(2)
                obpr.cd_suma    = (cd_cena * mn).round(2)
            }
            obpr.insert()
        }
        obpr.close()
    }

    // Rezervacia jednej objednavky
    private void reservation (tangram.SQLconn sql, String obphxx)
    {
        // Z hlavicky nacitame druh objednavky, z druhu objednavky sposob rezervacie, ak nie je zapnuta rezervacia tak nerobime nic
        def druhDok = sql.queryValueStr("SELECT Druhdok FROM OBPH WHERE Xx='$obphxx'")
        def rezimRezervacie = sql.queryValueStr("SELECT Rezerv FROM C_OBP WHERE Xx='$druhDok'")
        // Rezim rezervacie: A=disp.mnozstvo, B=cele mnozstvo, null=nerezervovat
        if (!rezimRezervacie) return

        // Prechadzame riadky, ktore este neboli rezervovane
        Log.info("REZERVÁCIA OBJEDNÁVKY $obphxx")
        def rows = sql.queryRows("""
            SELECT R.Xx, R.Sklad, R.Kod, nvl(R.Mnzost,0)-nvl(R.Mnrez,0) as MnPozadovane, T.Kategoria
            FROM OBPR R LEFT JOIN TOVARY T ON (T.Xx=R.Kod)
            WHERE (R.Xh='$obphxx') and nvl(R.Mnzost,0) > nvl(R.Mnrez,0) and (R.Sklad is not null)
            ORDER BY R.Xn""")
        while (rows.next())
        {
            def xx = rows.getString("xx")
            def kategoria = rows.getString('kategoria')
            def sklad = rows.getString('sklad')
            def kod = rows.getString('kod')
            def mnPozadovane = rows.getBigDecimal('MnPozadovane')
            def mnDisp = BigDecimal.ZERO
            // Rezim "CELE" alebo Sluzba alebo Log.zlava rezervuje cele zostavajuce mnozstvo
            if (rezimRezervacie=='B' || kategoria=='S' || kategoria=='L')
                mnDisp = mnPozadovane
            else
                mnDisp = sql.queryValueNum("SELECT Disp FROM SKDISP WHERE Sklad='$sklad' and Kod='$kod'").toBigDecimal()
            def mnRez = mnPozadovane.min(mnDisp)
            if (mnRez > 0)
            {
                sql.execute("UPDATE OBPR SET MnRez = nvl(MnRez,0) + $mnRez WHERE Xx='$xx'")
                Log.debug("rezervácia, kód=$kod, množstvo=$mnRez")
            }
        }
        rows.close()
    }
}

