import tangram.Log
import tangram.SQLconn
import tangram.TangramJsonWriter
import tangram.http.TangramHttpRequest

// zoznam tovarov s aktualnou cenou (GET), query-parametre:
// [id]=kod tovaru alebo [barcode]=ciarovy kod, vrati jeden zaznam
// [pricegroup] = cenova skupina
// ziaden parameter = export vsetkych tovarov s priznakom "e-shop"
urlPriceList (sql, request, json, cfg)

private static void urlPriceList (tangram.SQLconn sql, tangram.http.TangramHttpRequest request, tangram.TangramJsonWriter json, tangram.ConfigFile cfg)
{
    // Parametre z URL a konfiguracie (Tangram.ini)
    def qm = request.queryMap;
    def id = qm.getString('id').toUpperCase()
    def barcode = qm.getString('barcode').toUpperCase()
    def censkup = qm.getString('pricegroup', cfg.str('EshopPriceGroup', '001'))
    def typceny = cfg.str('EshopPriceBasic', "'A','B'")

    // Ak je parameter BARCODE, tak vyhladame prislusny kod tovaru
    // Ak sa nenajde ciarovy kod, nastavi sa ID='???' a vrati sa prazdny zoznam tovarov
    // Ak sa najde ciarovy kod vo viacerych tovaroch, vrati len jeden kod (najvyssi)
    if (barcode)
        id = sql.queryValueStr("SELECT nvl(max(Kod),'???') FROM TOVBARCODE WHERE Barcode='$barcode'")

    def tovfilter = 'T.Flg_eshop is not null' + (id ? " and T.Xx='${id}'" : '')
    Log.debug ("PRICELIST priceGroup=$censkup id=$id barcode=$barcode filter=$tovfilter")

    // API REQUEST: ciselnik sluzieb z tabulky TOVARY, optional filters: id=kod_tovaru
    // warning: parameter [id] sa z nejakeho dovodu neda pouzit vo where :id, musime urobit inject (skontrolovat SQL class)
    def select = """
    SELECT T.Xx "id", T.Druh "group", T.Nazov "name", T.Mj "unit", T.Mjbal "packunit", T.Balenie "package", T.Jhm "weight", T.Jobj "volume", T.Jmj "siunit", T.Jkoef "sicoef", 
		B.Barcode "barcode", 
        T.Znacka "trademark", T.Prodskup "prodgroup", T.Uct, T.Klasifikacia "classgroup", T.Certifikat "certificate", T.Vyrobca "manufacturer",
        T.Pozn "description", T.Zlozenie "ingredients", T.Fo_alergeny "alergens", T.Alkohol "alcohol", T.Povod "origincountry", T.Obal "retpack",
        C.Censkup "pricegroup", C.Typceny "pricetype", C.Sadzba_dph "pricevat", C.Cm_sdph "priceinc", C.Cm_bdph "priceexc", 
        C.Zl_typ "discounttype", C.Zl_t "discountvalue", C.Akcia "promotion", T.Povsort "sortgroup"
    FROM TOVARY T
    LEFT JOIN 
    ( SELECT T.Xx as Tovxx, max(C.Xx) KEEP (DENSE_RANK FIRST ORDER BY C.Cm_bdph*(1-nvl(C.Zl_t,0)/100) asc, C.Datum_od desc) as Cenxx
      FROM TOVARY T LEFT JOIN TOVCENY C ON (C.Kod=T.Xx)
      WHERE $tovfilter and C.Censkup='$censkup' and C.Typceny in ($typceny) and (trunc(sysdate) between C.Datum_od and C.Datum_do) 
      GROUP BY T.Xx
    ) X ON (T.Xx=X.Tovxx)
    LEFT JOIN TOVCENY C ON (C.Xx=X.Cenxx)
    LEFT JOIN TOVBARCODE B ON (B.Kod=T.Xx and B.Hlavny is not null)
    WHERE $tovfilter
    ORDER BY T.Xx
    """

    def rows = sql.queryRows(select)
    Log.debug ("Dataset nacitany z databazy")

    // Response header
    request.sendHeaderJSON()

    // Reponse body
    json.begin()
    json.resultset(rows, 'items')
    json.end()
    rows.close()
    Log.debug ("Dataset zapisany do JSON")
}
