package tangram

import tangram.http.HTTPSClient
import tangram.http.TangramHttpRequest

def worker = new CliOrdersWorker(request, sql, json)
worker.run()

/**
 * PPG ClientLink interface - Product send (skript)
 *   COUNTRY=CZ alebo SK
 *   TEST=TRUE alebo FALSE (default)
 *   MAX=nnn (max.pocet exportovanych zaznamov, default=1000)
 * Export dat z tabulky TOVARY, pridane stlpce X_CRM, X_CRMID, X_CRMSENT, X_CRMRESULT
 * Pre kazdy tovar sa vytvori samostatna SOAP-sprava a odosle sa na server
 */
class CliOrdersWorker
{
    // Premenne naplnene constructorom
    TangramHttpRequest httpRequest
    SQLconn sql
    TangramJsonWriter json
    // Parametre requestu
    String country
    boolean test
    int max = 0
    // Pripojenie do ClientLink cez SOAP
    String url
    String user
    String password
    String template

    // Constructor
    CliOrdersWorker (TangramHttpRequest httpRequest, SQLconn sql, TangramJsonWriter json)
    {
        this.httpRequest = httpRequest
        this.sql = sql
        this.json = json

        Log.info("Client Link - Orders export")
        def params = this.httpRequest.queryMap
        country = params.getString('country')
        test = params.getBoolean('test', true)
        max = params.getInt('max', 1000)
        if (country!='SK' && country!='CZ' && country!='XX')
            throw new TangramException('Bad request, required parameter COUNTRY=CZ or COUNTRY=SK')

        // Mapovanie SK->PPGSVK, CZ->TRIGA, budeme potrebovat nastavit rozne hodnoty pre SK/CZ a pre TEST/PROD
        String schema = ''
        if (country=='CZ') schema = test ? 'TESTCZ' : 'TRIGA'
        if (country=='SK') schema = test ? 'TESTSK' : 'PPGSVK'
        if (country=='XX') schema = 'EKO'
        def info = "Setting schema to $schema"
        Log.info(info)
        try { sql.conn.setSchema (schema) }
        catch (Exception e) { throw new TangramException(info, e) }

        // nastavenie pripojenia do CLI
        // def url = "https://e100211-iflmap.hcisbt.eu1.hana.ondemand.com:443/cxf/tangram/orders/dev"
        url = test ?
                'https://e100211-iflmap.hcisbt.eu1.hana.ondemand.com:443/cxf/tangram/orders/qa' :
                'https://l100431-iflmap.hcisbp.eu1.hana.ondemand.com:443/cxf/Tangram/orders/prd'
        user     = test ? 'S0021493398' : 'S0021493400'
        password = test ? 'Um%`.2Jr' : '!z96f$CJ'
        template = (country=='CZ') ? 'PPG/CliOrderCZ.ftlx' : 'PPG/CliOrderSK.ftlx'
    }

    // Odoslanie vsetkych dokladov s novsim timestampom (kategoria dokladu Predaj, firma musi byt vyplnena)
    void run ()
    {
        // Vyhladanie dokladov, ktore este neboli odoslane
        def select = """
        SELECT H.Xx, H.Xid, H.Firma, H.Druhdok, H.Cdok, F.X_crmid, count(*) over() as Pocet
        FROM SKLADH H 
		LEFT JOIN FIRMY F ON (F.Firma || '.'|| F.Pobocka = H.Firma || '.' || H.Pobocka)
        WHERE (H.Datum >= trunc(sysdate-60)) and H.X_crmsent is null and H.Kategoria='D' and H.Firma is not null
        ORDER BY H.Datum, H.Druhdok, H.Cdok
        """.stripIndent()

        def rs = sql.queryRows(select)

        // Generovanie odpovede pre volajuceho, po kazdom zazname dopiseme a flushneme aby nedoslo k timeoutu
        httpRequest.sendHeaderJSON()
        json.begin()

        def counter = 1
        while (rs.next()) {
            def xx = rs.getString("xx")
            def pocet = rs.getInt("pocet")
            def data = new DataBuffer(rs.data)
            // k hlavicke pridame pole pre riadky
            def rowArray = new ArrayList()
            data.put("rows", rowArray)

            // Pre kazdy doklad nacitame riadky
            def sumaZaDoklad = new BigNumber(0)
            def rownum = 0
            def skladr = sql.queryRows("""
            SELECT R.Xn, R.Kod, R.Mnv, R.Cm_suma, T.Nazov, T.Mj, T.Druh
            FROM SKLADR R LEFT JOIN TOVARY T ON (T.Xx=R.Kod)
            WHERE R.Xh='$xx' and T.Kategoria in ('T','V')
            ORDER BY R.Xn
        """.stripIndent())
            while (skladr.next())
            {
                def row = new DataBuffer(skladr.data)
                row.rownum = rownum
                row.mj = convertUnit(row.getString('mj'))
                rowArray.add(row)
                sumaZaDoklad += row.cm_suma
                rownum++
            }
            skladr.close()

            data.sumaZaDoklad = sumaZaDoklad
            Log.newline()
            Log.info("Sending ORDER message ($counter/$pocet), SKLADH.XX=$xx ${data.druhdok}.${data.cdok}")
            Log.debug("${data}")

            // odoslanie jedneho dokladu
            sendDoc(data)

            // po kazdom doklade pridame riadok do vystupu, aby klient nevyhodil timeout
            json.pair(xx, "${data.druhdok}.${data.cdok}")
            json.flush()

            counter++
            if (counter > max) break
        }
        rs.close()
        json.end()

        if (counter==1) Log.info ('No new DOCUMENTS found')
    }

    // Odoslanie jedneho dokladu, formatovanie spravy cez FreeMarker (Object HTML)
    private void sendDoc(DataBuffer data)
    {
        def timeStart = System.currentTimeMillis()
        def body = HTML.process(data, template)
        def timeTemplate = System.currentTimeMillis()

        // Odoslanie zaznamu do ClientLink
        def response = sendSOAP(body)
        def timeSending = System.currentTimeMillis()
        def xx = data.getString('xx')
        Log.info("ORDER message sent, ID=$xx, formatting=${timeTemplate-timeStart}ms, sending=${timeSending-timeTemplate}ms")

        // Zapis odpovede do nasej databazy
        saveResponse(response, xx)
    }

    // Odoslanie spravy na HTTP server (body obsahuje naformatovanu SOAP spravu)
    // Vracia Response-body, aby sme mohli analyzovat vysledok
    private String sendSOAP (String body)
    {
        Log.debug("HTTP REQUEST:$url\n$body")
        // PPG CRM connect
        // TODO >> mozne zrychlenie, ak HTTPS klienta inicializujeme len raz na zaciatku ?
        def client = new HTTPSClient(url)
        client.setHeader("Accept", "text/xml")
        client.setHeader("Content-Type", "text/xml;charset=utf-8")
        client.setHeader("User-Agent", "Tangram")
        client.setBasicAuth(user, password)
        // Odoslanie
        client.post(body)
        //
        Log.debug("HTTP RESPONSE:\n${client.responseText}")
        Log.debug("RESPONSE HEADERS:\n${client.connection.headerFields}")
        return client.responseText
    }

    // Zapis vysledku do databazy (response sa zatial neanalyzuje)
    private saveResponse (String response, String skladhXX)
    {
        sql.execute("UPDATE SKLADH SET X_crmsent=sysdate WHERE Xx='$skladhXX'")
    }

    // Konverzia mernych jednotiek na zvlastny PPG ciselnik, default je ZPL (piece/kus)
    private static String convertUnit(String mj)
    {
        switch (mj.toUpperCase())
        {
            case 'KS': return 'Z65'
            case 'KG': return 'KGM'
            case 'L':  return 'LTR'
            case 'BAL':return 'XPK'
            default: return 'Z65'
        }
    }
}
