import businessLogic.ppg.XpShopCode
import tangram.CfgApp
import tangram.ConfigFile
import tangram.DataBuffer
import tangram.Log
import tangram.SQLconn
import tangram.SQLfilter
import tangram.TangramJsonWriter
import tangram.http.TangramHttpRequest

// CUSTOMER LIST, zoznam zakaznikov
// Parametre requestu: ID=kod zakaznika alebo DAYS=xxx alebo HOURS=xxx (priorita v tomto poradi)
urlCustomerList(sql, request, json)

// Zoznam skladov (predajni) - exportuju sa vsetky okrem velkoskladov
private static void urlCustomerList(SQLconn sql, TangramHttpRequest request, TangramJsonWriter json)
{
    json.replaceNull = true

    def prm = request.queryMap
    def id = prm.getString('id')
    def days = prm.getInt('days', -1)
    def hours = prm.getInt('hours', -1)
    def country = CfgApp.eshopCountry

    // Response header
    request.sendHeaderJSON()

    if (id)
        sendOneCustomer(sql, json, id, country)
    else
        sendAllCustomers(sql, json, days, hours, country)
}

// Odoslanie jedneho zakaznika podla ID
private static void sendOneCustomer (SQLconn sql, TangramJsonWriter json, String id, String country)
{
    def firma = sql.querySingle ("SELECT F.*, A.Nazov2 as Xp_name FROM FIRMY F LEFT JOIN ADRESA A ON (A.Xx=F.Xp) WHERE F.Xx='$id'")
    def dataAPI = new DataBuffer()
    customerToAPI (firma, dataAPI, country)
	
    // Reponse body: zapisujeme len do bufferu, nesmieme urobit flush, to za nas urobi handler
    json.begin()
    json.map (dataAPI, 'customer')
    json.end()
}

// Odoslanie zoznamu zakaznikov podla casovej peciatky
private static void sendAllCustomers (SQLconn sql, TangramJsonWriter json, int days, int hours, String country)
{
    def filter = new SQLfilter()
    filter.add('F.Flg_eshop is not null')

    if (days >= 0) filter.add("nvl(F.Xud,F.Xid)>=(trunc(sysdate)-$days)")
    else if (hours > 0) filter.add("nvl(F.Xud,F.Xid)>=(sysdate - interval '$hours' hour)")

    def rows = sql.queryRows ("SELECT F.*, A.Nazov2 as Xp_name FROM FIRMY F LEFT JOIN ADRESA A ON (A.Xx=F.Xp) WHERE $filter ORDER BY F.Xx")

    // Response body
    json.begin()
    json.arrayBegin("rows")
    while (rows.next())
    {
        def dataAPI = new DataBuffer()
        customerToAPI(rows.data, dataAPI, country)
        json.map (dataAPI)
        json.flush()
        Log.debug ("CUSTOMER: $dataAPI")
    }
    json.arrayEnd()
    json.end()

    rows.close()
}

// Prevod CUSTOMER z databazy do API
private static void customerToAPI (DataBuffer db, DataBuffer api, String country)
{
    // Customer_type: anonymny zakaznik s kodom ESHOP.000000 = null, B2B = 1, B2C = 0
    def customerId  = db.xx
    def customerType  = (customerId=='ESHOP.000000') ? null : ((db.f=='P' || db.f=='F') ? 1 : 0)
    def xp = XpShopCode.addPrefix(db.getString("xp"), country)      // Prefix statu
    // boolean customerObcan   = customerType==null || customerType==0
    api.id                  = customerId
    api.customer_type       = customerType
    api.valid               = db.neaktivna ? 0 : 1
	api.company_name        = db.nazov1 + ' ' + db.nazov2 							                       //doplněno
    api.company_first_name  = db.nazov1
    api.company_last_name   = db.nazov2
    api.customer_name       = db.delivery_firstname + ' ' + db.delivery_lastname                           //doplněno
    api.siret               = db.ico
    api.vat                 = db.icdph
    api.email               = db.flg_eshop
    api.phone               = db.telefon
    api.invoice_addr1       = db.ulica ?: 'none'
    api.invoice_addr2       = ''        // customerObcan ? '' : db.nazov2
    api.invoice_post        = db.psc ?: '000 00'
    api.invoice_city        = db.mesto ?: 'none'
    api.invoice_country     = db.stat ?: country
    api.delivery_company    = db.delivery_company
    api.delivery_firstname  = db.delivery_firstname
    api.delivery_lastname   = db.delivery_lastname
    api.delivery_addr1      = db.delivery_addr1 ?: 'none'
    api.delivery_addr2      = db.delivery_addr2
    api.delivery_post       = db.delivery_post ?: 'none'
    api.delivery_city       = db.delivery_city ?: 'none'
    api.delivery_country    = db.delivery_country

    api.customer_group      = db.odbskup
    api.price_group         = db.censkup
    api.discount            = db.zlava
    api.default_shop_id     = xp ?: 'none'
    api.default_shop_name   = db.xp_name  ?: 'none'
    api.payment_methods     = db.getString("spuhrady").contains('P') ? 'PHDVX' : 'HDVX'
    api.delivery_options    = db.x_delivery ?: 'A'
    api.sales_rep_id        = db.obchzast
    api.sales_rep_phone     = null
    api.loyalty_program     = 0
    api.loyalty_points      = 0
    api.timestamp           = db.getTime('xud') ?: db.getTime('xid')
}

