import businessLogic.ppg.XpShopCode
import tangram.DataBuffer
import tangram.Log
import tangram.SQLconn
import tangram.SQLtable
import tangram.http.TangramHttpRequest

// Update udajov pre existujuceho zakaznika
// E-shop to vola len pre B2C zakaznikov, kod pobocky nie je vyplneny
urlCustomerUpdate(sql, request)

// Zoznam skladov (predajni) - exportuju sa vsetky okrem velkoskladov
private static void urlCustomerUpdate(SQLconn sql, TangramHttpRequest request)
{
    // CustomerID je v QUERY, vstupne data su v request.bodyMap (rozparsovany JSON)
    def body = request.bodyMap
    // Pripravime tabulku, nacitame povodny zaznam z databazy, ak sa nenajde vrati sa chyba
    def id = request.queryMap.getString("id")
    if (id.isEmpty()) { request.sendError(id, 'IDNULL', "Customer ID not specified"); return }
    int count = sql.queryValueInt("SELECT count(*) FROM FIRMY WHERE Xx='$id'")
    if (count==0) { request.sendError(id, 'NOTFOUND', "Kód zákazníka $id nenájdený v databáze"); return }

    // def b2b = !(row.getString('f') == 'O')
    // Update e-shop kontaktu (ak su zadane prislusne polia)
    updateContact (sql, id, body)

    // Vytiahneme vsetky hodnoty z request-body, naplnime body->upd
    body.remove("phone")
    body.remove("email")
    def upd = new DataBuffer()
    def errorList = createBody (upd, body)

    // Nespracovane polia vyhodia chybu
    if (!errorList.isEmpty()) // { request.sendError(id, 'DENIED', "Access denied: $errorList"); return }
        Log.error('UPDATE nepovolené polia ' + errorList)

    // Update zaznamu v databaze
    def f = new SQLtable ('FIRMY', sql)
    f.update(id, upd)
    f.close()

    // koniec OK
    Log.info("UPDATE CUSTOMER, id=$id")
    request.sendOk(id, 'Zákazník zmenený v databáze')
}

// Naplnenie stlpcov tabulky FIRMY, transformuju sa nazvy, kontroluju sa povolene stlpce pre kazdy rezim
// [upd] = zoznam poli v tabulke FIRMY, ktore sa maju updatovat (return value)
// [body] = zoznam poli z requestu
// Vrati: zoznam chyb alebo prazdny retazec ak je vsetko OK
private static String createBody (DataBuffer upd, DataBuffer body)
{
    // B2B: true=firma, false=obcan
    def errorList = ''
    // zoznam povolenych poli, preklad na databazove nazvy
    // 04.05.2023 polia email a phone sa nezapisuju do FIRMY ale do kontaktu
    def translate = [
            company_name: 'nazov1', invoice_addr1: 'ulica', invoice_addr2: 'nazov2', invoice_post: 'psc', invoice_city: 'mesto', invoice_country: 'stat',
            delivery_company: 'delivery_company', delivery_firstname: 'delivery_firstname', delivery_lastname: 'delivery_lastname',
            delivery_addr1: 'delivery_addr1', delivery_addr2: 'delivery_addr2',
            delivery_post: 'delivery_post', delivery_city: 'delivery_city', delivery_country: 'delivery_country',
            default_shop_id: 'xp']
    // ak nie je B2B, pridame k povolenym aj nazov a adresu firmy
    //if (!b2b) translate << [company_name: 'nazov1', invoice_addr1: 'ulica', invoice_addr2: 'nazov2',
    //                        invoice_post: 'psc', invoice_city: 'mesto', invoice_country: 'stat']
    // ak je OBCAN a meni sa Meno aj Priezvisko, tak prepiseme aj Nazov firmy
    // if (!b2b && body.delivery_firstname && body.delivery_lastname)
    //   body.company_name = body.delivery_firstname + ' ' + body.delivery_lastname
    // do [upd] vytvorime zoznam databazovych stlpcov s novymi hodnotami
    body.each { key, value ->
        def dbColumn = translate.get(key)
        // Kod pobock-y: orezanie prefixu
        if (dbColumn == "xp") value = XpShopCode.removePrefix(value.toString(),'')
        Log.debug("$key ($value) -> $dbColumn")
        if (!dbColumn) errorList += "$key "
        else upd.put(dbColumn, value)
    }

    return errorList
}

// Update kontaktu (prvy kontakt s priznakom "eshop")
private static void updateContact (SQLconn sql, String firma, DataBuffer body)
{
    // Skratka: B2C nema pobocky a stlpec FIRMA = XX, nemusime nacitavat zaznam z tabulky FIRMY
    // Vyhladame polia pre FIRKONTAKT, ak nie je ziadne zadane taknic nezapisujeme
    def upd = new DataBuffer();
    if (body.containsKey("delivery_firstname"))
        upd.meno = body.getString("delivery_firstname")
    if (body.containsKey("delivery_lastname"))
        upd.priezvisko = body.getString("delivery_lastname")
    if (body.containsKey("phone"))
        upd.mobil = body.getString("phone")
    if (body.containsKey("email"))
        upd.email = body.getString("email")
    if (upd.isEmpty()) return

    // Vyhladame ID zaznamu vo FIRKONTAKT, ak sa nenajde tak nic nezapisujeme
    String xx = sql.queryValueStr("SELECT min(xx) FROM FIRKONTAKT WHERE firma='$firma' and f_eshop is not null")
    if (! xx)
    {
        Log.debug("FIRKONTAKT: no e-shop contact found for customer $firma")
        return
    }

    // Update zaznamu v databaze
    def k = new SQLtable ('FIRKONTAKT', sql)
    k.update(xx, upd)
    k.close()
    Log.debug("FIRKONTAKT: contact [$xx] updated, values=$upd")
}
