import businessLogic.SkPrice
import businessLogic.ppg.PpgOrderRowP
import businessLogic.ppg.PpgOrderRowT
import tangram.CfgApp
import tangram.JsonWriter
import tangram.Log
import tangram.Num
import tangram.SQLconn
import tangram.http.TangramHttpRequest

import java.time.LocalDate

// PRICE - vyhladanie ceny pre jeden produkt (ak je zadane pole COLORANTS, je to miesana farba
// parametre su v tele requestu (POST); CUSTOMER_ID, ITEM_ID a pole COLORANTS
urlPrice (sql, request)

// Pomocna trieda pre naformatovanie vysledku (Jackson z toho generuje JSON)
class PriceResult
{
    public String result = 'OK'
    public String customer_name
    public String price_group
    public BigDecimal discount
    public String item_id
    public BigDecimal priceb_bv
    public BigDecimal priceb_iv
    public BigDecimal priced_bv
    public BigDecimal priced_iv
    public BigDecimal vat
    public String promotion
    public BigDecimal price_discount
    public String price_type
    public LocalDate valid_from
    public LocalDate valid_to
    public String comment
    // Pole promo-cien podla mnozstiev, my budeme mat vzdy maximalne jednu promo cenu s mnozstvom od 1
    public ArrayList<PricePromo> price_promo = new ArrayList<>();

    // Naplnenie poli z vyhladanej ceny
    public void loadPrice (SkPrice price)
    {
        priceb_bv  = price.cena_bdph
        priceb_iv  = price.cena_sdph
        priced_bv  = price.zlav_bdph
        priced_iv  = price.zlav_sdph
        vat        = price.sadzba_dph
        promotion  = price.akcia
        price_discount = price.zlava
        price_type = price.typceny
        valid_from = price.datum_od
        valid_to   = price.datum_do
        comment    = price.popis
    }
}

// Definicia objektu pre vlozenie Promo-ceny
class PricePromo
{
    public BigDecimal price_bv;
    public BigDecimal price_iv;
    public BigDecimal price_quantity;

    PricePromo(BigDecimal price_bv, BigDecimal price_iv)
    {
        this.price_bv = price_bv;
        this.price_iv = price_iv;
        price_quantity = BigDecimal.ONE;
    }
}

// Funkcia na vypocet ceny
private static void urlPrice (SQLconn sql, TangramHttpRequest request)
{
    // Header: podla kodu zakaznika vyhladame Cenovu skupinu a Dokladovu zlavu
    def body = request.bodyMap
    def customerId = body.getString('customer_id')
    def itemId = body.getString('item_id')
    if (!itemId) { request.sendError('', 'IDNULL', "ITEM ID not specified"); return }
    def colorants = body['colorants']
    def priceGroupDefault = CfgApp.eshopPriceGroup
    def priceGroupOverride = CfgApp.eshopPriceOverride

    // Vyhladanie zakaznika, ak sa nenajde pouziju sa default hodnoty
    // simulujeme OrderHead, lebo cez hlavicku sa odovzdaju parametre na hladanie ceny
    def firma = sql.querySingle("SELECT * FROM FIRMY WHERE Xx='$customerId'")
    // parametre pre vypocet ceny
    def prm = new SkPrice.SkPricePrm();
    prm.pKod = itemId
    prm.pDatum = LocalDate.now()
    prm.pFirma = firma.getString('firma')
    prm.pPobocka = firma.getString('pobocka')
    prm.pCenskup = priceGroupOverride ? priceGroupDefault : (firma.getString('censkup') ?: priceGroupDefault)
    prm.pZlDoklad = firma.getBigDecimal('zlava')
    prm.pPredajna = firma.getString('xp')

    // Miesto pre vysledok operacie, predplnia sa niektore polia
    def result = new PriceResult()
    result.customer_name = firma.getString('nazov1')
    result.price_group = prm.pCenskup
    result.discount = prm.pZlDoklad
    result.item_id = itemId

    // simulujeme OrderRow, bud produkt alebo miesana farba
    Log.debug("PRICE: item=$itemId customer=$customerId pricegroup=${prm.pCenskup} cust.discount=${prm.pZlDoklad} xp=${prm.pPredajna}")
    if (!colorants)
    {
        // Cena pre standardny produkt, najprv sa ulozi do poli pre zakladnu cenu
        def row = new PpgOrderRowP()
        row.item_id = itemId
        row.loadPrice(sql, prm)
        // Kontrola chyb pre PRODUCT, nenajdena cena
        if (!row.price.found)
        {
            request.sendError(itemId, 'PRICENULL', "ITEM ${itemId}, group ${prm.pCenskup}, price not found")
            return
        }
        Log.debug(row.price.toString())
        result.loadPrice(row.price)

        // Vyhladanie zakladnej ceny (typ "A")
        def basicPrice = new SkPrice();
        basicPrice.findBasicPrice(sql, prm);
        Log.debug("BASIC PRICE: " + basicPrice);

        // Ak sme nasli zakladnu cenu a je vyssia ako najdena, tak ju pouzijeme
        if (basicPrice != null && Num.compare(basicPrice.zlav_bdph, result.priced_bv)>0)
        {
            // predtym vypocitanu cenu presunieme do pola price_promo (bude tam len jeden prvok)
            result.price_promo.add(new PricePromo(result.priced_bv, result.priced_iv));
            // zakladnu cenu vlozime namiesto predtym vypocitanej
            result.priceb_bv = basicPrice.cena_bdph;
            result.priceb_iv = basicPrice.cena_sdph;
            result.priced_bv = basicPrice.zlav_bdph;
            result.priced_iv = basicPrice.zlav_sdph;
        }
        // result.loadPromoPrice(basicPrice);    // test, vlozime najdenu cenu
    }
    else
    {
        // Cena pre miesanu farbu (tu sa nepocita "promo" cena)
        def row = new PpgOrderRowT()
        row.item_id = itemId
        row.parseColorants(sql, colorants)
        row.loadPrice(sql, prm)
        // Kontrola cien
        if (!row.basePrice.found) { request.sendError(itemId, 'PRICENULL', "COLOR BASE ${itemId}, price not found"); return }
        for (c in row.colorants)
        {
            if (c.getSku().isEmpty()) { request.sendError(itemId,'COLIDNULL',"COLORANT ${c.getColorId()} not found in table (X_COLORCODE)"); return }
            if (!c.getPrice().found) { request.sendError(itemId,'COLPRICENULL',"COLORANT ${c.getColorId()} (${c.getSku()}), price not found"); return }
        }
        Log.debug(row.price.toString())
        result.loadPrice(row.price)
        // cena miesanej farby uz zohladnuje vsetky promo akcie a nepocita sa samostatna promo-cena, pole zostane prazdne
    }

    // Response header + body
    request.sendHeaderJSON()
    JsonWriter.write(request.writer, result)
}

//private static SkPrice findBasicPrice (SQLconn sql, String )


