import businessLogic.SkUtils
import businessLogic.TN_MSG
import tangram.DataBuffer
import tangram.Log
import tangram.SQLconn
import tangram.SQLtable
import tangram.http.TangramHttpRequest

import java.time.LocalDate
// QUOTE ACCEPT: prijatie cenovej ponuky, JSON, UTF-8
// data cez POST
execute(sql, request)

private static void execute(SQLconn sql, TangramHttpRequest request)
{
    def data = request.bodyMap
    def id = data.getString('quote_id')

    // Kontroly
    if (id.size() != 16) { request.sendError(id, "ILLEGAL", "Illegal ID format (16 chars)"); return}
    def cpHead = sql.querySingle("SELECT * FROM OBCPH WHERE Xx='$id'")
    if (! cpHead.xx) { request.sendError(id, "NOTFOUND", "Unknown quote ID"); return }
    if (cpHead.stav != 'A') { request.sendError(id, "NOTOPEN", "Quote is not opened"); return }
    // Log.debug("Cenová ponuka = " + cpHead)

    // skopirovat hlavicku ponuky (data v mape cpHead) do objednavky
    def obph = new SQLtable("OBPH", sql)
    def obHead = createHeader(sql, cpHead, data)
    obph.copyFrom(obHead)
    obph.insert()
    def Xh = obph.data.getString("Xx")
    Log.debug("OBPH.XX=$Xh")

    // skopirovat riadky objednavky
    copyRows(sql, id, Xh)

    // Oznacit ponuku za AKCEPTOVANU
    sql.execute("UPDATE OBCPH SET Stav='B', Obphxx='$Xh', D_potvrd=trunc(sysdate) WHERE Xx='$id'")

    // Nova objednavka - notifikacia mailom
    TN_MSG.write(sql, "OrderNotification", Xh)

    // Odoslanie systemovej spravy pre automat, ktory vystavi fakturu k prijatej platbe
    if ("V" == obHead.sp_uhrady)
        TN_MSG.write(sql, "OrderPayment", Xh);

    // TODO >> este chyba odoslanie notifikacie pre PCkasy, na ktorych sa moze objednavka spracovat

    // koniec OK
    Log.info("QUOTE ACCEPTED, NEW ORDER CREATED id=$Xh")
    request.sendOk('ESHOP/' + obHead.cdok, 'Quote accepted')
}

// Kopirovanie riadkov ponuky do objednavky
private static void copyRows (SQLconn sql, String quoteXx, String orderXx)
{
    def cpRows = sql.queryRows("SELECT * FROM OBCPR WHERE Xh='$quoteXx' ORDER BY Xn")
    def obpr = new SQLtable("OBPR", sql)
    while (cpRows.next())
    {
        obpr.clear()
        obpr.data.copyFrom(cpRows.data)
        obpr.set("Xx", null)
        obpr.set("Xh", orderXx)
        obpr.set("Zavazna", "A")
        obpr.set("Mnzost", cpRows.getBigDecimal("Mn"))
        obpr.insert()
    }
}

// Vytvorenie hlavicky objednavky podla hlavicky ponuky
// cpHead = hlavicka povodnej ponuky (map)
// reqData = request body, zmenene data z e-shopu (dodacia adresa a podobne)
// RETURN hlavicka novej objednavky (map)
private static DataBuffer createHeader(SQLconn sql, DataBuffer cpHead, DataBuffer reqData)
{
    // skopirovanie hlavicka ponuky do novej mapy 1:1
    def oh = new DataBuffer(cpHead)

    // nastavenie zakladnych poloziek v hlavicke novej objednavky
    oh.Xx = null
    oh.Druhdok = 'ESHOP'  // 07.09.2021 OBP budu natvrdo zaradene medzi e-shop objednavky
    oh.Cdok = sql.queryMaxPlus('OBPH', 'Cdok', "Druhdok='${oh.Druhdok}'")
    oh.Datum   = LocalDate.now()
    oh.Obcphxx = cpHead.xx
    oh.Rsstav  = '1'
    oh.Extcis  = "${cpHead.Druhdok}/${cpHead.Cdok}"

    // Kod firmy (xx) sa musi rozdelit na Firma + Pobocka
    def firmaxx = cpHead.getString("firmaxx")
    oh.Firma   = SkUtils.firmaFirma(firmaxx)
    oh.Pobocka = SkUtils.firmaPobocka(firmaxx)

    // doplnenie udajov z requestu (zadane cez e-shop pri potvrdzovani ponuky)
    if (reqData.containsKey('payment')) oh.sp_uhrady = reqData.payment
    if (reqData.containsKey('payinfo')) oh.vs_uhrady = reqData.payinfo
	if (reqData.containsKey('comment')) oh.text = reqData.comment

    return oh
}
